//////////////////////////////////////////////////////////////////////////
//
// BMP2AVIDlg.cpp : implementation file
// Copyright : Yusupova Julia;  email: julus@yandex.ru
//
//////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BMP2AVI.h"


#include "BMP2AVIDlg.h"
#include <atlbase.h>

#include <initguid.h>
#include "PictureGUIDs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBMP2AVIDlg dialog

CBMP2AVIDlg::CBMP2AVIDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBMP2AVIDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBMP2AVIDlg)
	m_bmp = _T("");
	m_avi = _T("");
	m_FrameRate = 10;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	CoInitialize(NULL);
	m_pGraphBuilder = NULL;
	m_pMediaEvent = NULL;
	m_pMediaControl = NULL;
}

CBMP2AVIDlg::~CBMP2AVIDlg()
{
	ReleaseAll();
	CoUninitialize();
}

HRESULT CBMP2AVIDlg::ReleaseAll()
{
	SAFE_RELEASE(m_pMediaControl);
	if (m_pMediaEvent){
		m_pMediaEvent->SetNotifyWindow(NULL, WM_GRAPHNOTIFY, 0);
		m_pMediaEvent->Release();
		m_pMediaEvent = NULL;
	}
	SAFE_RELEASE(m_pGraphBuilder);
	return S_OK;
}

void CBMP2AVIDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBMP2AVIDlg)
	DDX_Text(pDX, IDC_BMP, m_bmp);
	DDX_Text(pDX, IDC_AVI, m_avi);
	DDX_Text(pDX, IDC_FRAMERATE, m_FrameRate);
	DDV_MinMaxUInt(pDX, m_FrameRate, 1, 1000);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CBMP2AVIDlg, CDialog)
	//{{AFX_MSG_MAP(CBMP2AVIDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_CHECK_BMP, OnCheckBmp)
	ON_BN_CLICKED(IDC_CHECK_AVI, OnCheckAvi)
	ON_MESSAGE(WM_GRAPHNOTIFY, onGraphNotify)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBMP2AVIDlg message handlers

BOOL CBMP2AVIDlg::OnInitDialog()
{
	CWinApp * app = AfxGetApp();
	m_bmp = app->GetProfileString(_T("BMP2AVI"),_T("bmpfile"));
	m_avi = app->GetProfileString(_T("BMP2AVI"),_T("avifile"));
	
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CBMP2AVIDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CBMP2AVIDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CBMP2AVIDlg::OnCheckBmp() 
{
	CFileDialog dlg(TRUE);
	dlg.m_ofn.lStructSize = sizeof(OPENFILENAME);
	CString filter;
	filter = _T("BMP files (*.bmp)");
	filter += (TCHAR)NULL;
	filter += _T("*.bmp");
	filter += (TCHAR)NULL;
	dlg.m_ofn.lpstrFilter = filter;
	dlg.m_ofn.nFilterIndex = 1;
	dlg.m_ofn.lpstrInitialDir = m_bmp;
	if (dlg.DoModal()==IDOK) {
		m_bmp = dlg.GetPathName();
		GetDlgItem(IDC_BMP)->SetWindowText(m_bmp);
	}	
}

void CBMP2AVIDlg::OnCheckAvi() 
{
	CFileDialog dlg(FALSE);
	dlg.m_ofn.lStructSize = sizeof(OPENFILENAME);
	CString filter;
	filter = _T("MPEG files (*.avi)");
	filter += (TCHAR)NULL;
	filter += _T("*.avi");
	filter += (TCHAR)NULL;
	
	dlg.m_ofn.lpstrFilter = filter;
	dlg.m_ofn.nFilterIndex = 1;
	dlg.m_ofn.lpstrInitialDir = m_avi;
	if (dlg.DoModal()==IDOK) {
		m_avi = dlg.GetPathName();
		GetDlgItem(IDC_AVI)->SetWindowText(m_avi);
	}	
}

void CBMP2AVIDlg::OnCancel() 
{
	ReleaseAll();
	UpdateData();
	CWinApp * app = AfxGetApp();
	app->WriteProfileString(_T("BMP2AVI"), _T("bmpfile"), m_bmp);
	app->WriteProfileString(_T("BMP2AVI"), _T("avifile"), m_avi);
	
	CDialog::OnCancel();
}

void CBMP2AVIDlg::OnOK() 
{
	// if bmp file is rendering now, then stop process
	if (m_pGraphBuilder){
		ReleaseAll();
		GetDlgItem(IDOK)->SetWindowText(_T("Start"));
		return;
	}

	if (!UpdateData())
		return;

	// The BMP file must be pointed
	if (m_bmp.IsEmpty()){
		MessageBox(_T("Set BMP file in this field"));
		GetDlgItem(IDC_BMP)->SetFocus();
		return;
	}
	// The BMP file must exist
	WIN32_FIND_DATA FindFileData;
	HANDLE hFind = FindFirstFile( m_bmp, &FindFileData);
	if (INVALID_HANDLE_VALUE == hFind){
		CString str;
		str.Format(_T("File %s not found."),m_bmp.LockBuffer());
		m_bmp.UnlockBuffer();
		MessageBox(str);
		GetDlgItem(IDC_BMP)->SetFocus();
		return;
	}
	FindClose(hFind);

	// If AVI file is not pointed then render in the screen
	if (m_avi.IsEmpty()) 
		m_out = OUT_ON_SCREEN;
	else {
		m_out = OUT_IN_AVI;
		// delete AVI file if it exist
		hFind = FindFirstFile( m_avi, &FindFileData);
		if (INVALID_HANDLE_VALUE != hFind){
			FindClose(hFind);
			if (!DeleteFile(m_avi)){
				CString str;
				str.Format(_T("Cannot delete file %s. \nTry to input another avi file."), m_avi.LockBuffer());
				m_avi.UnlockBuffer();
				MessageBox(str);
				return;
			}
		}
		else{
			DWORD dwerr = GetLastError();
			if (ERROR_FILE_NOT_FOUND != dwerr){
				CString str;
				str.Format(_T("Cannot open file %s. \nTry to input another avi file."), m_avi.LockBuffer());
				m_avi.UnlockBuffer();
				MessageBox(str);
				return;
			}
		}
	}
	// Build and run graph
	HRESULT hres;
	if (FAILED(hres = BuildGraph())){
		ReleaseAll();
		if (hres == VFW_E_CANNOT_RENDER) MessageBox(_T("Cannot render graph."));
		else MessageBox(_T("Cannot build graph"));
	}
	
}

HRESULT CBMP2AVIDlg::BuildGraph()
{
	ReleaseAll();
	
	if (m_bmp.IsEmpty() || m_out==OUT_IN_AVI&&m_avi.IsEmpty())
		return E_FAIL;
	
	//HRESULT hr;
	
	JIF(CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER,
		IID_IGraphBuilder,(void **)&m_pGraphBuilder));
	
	// Add Source File
	CComPtr<IBaseFilter> pSource;
	JIF(pSource.CoCreateInstance(CLSID_BMP2Stream));
	JIF(m_pGraphBuilder->AddFilter(pSource,_T("BMP Picture")));
	
	CComPtr <IFileSourceFilter> pfsource;
	JIF(pSource->QueryInterface(IID_IFileSourceFilter,(void**)&pfsource));
	USES_CONVERSION;
	JIF(pfsource->Load((LPCOLESTR)m_bmp.GetBuffer(), NULL));

	// Set Frame rate in BMP2Stream filter
	CComPtr <IPictureSettings> pSettings;
	JIF(pSource->QueryInterface(IID_IPictureSettings,(void**)&pSettings));
	JIF(pSettings->put_FrameRate(m_FrameRate));

	// Add CSC
	CComPtr<IBaseFilter> pColorSpace;
	JIF(pColorSpace.CoCreateInstance(CLSID_Colour));
	JIF(m_pGraphBuilder->AddFilter(pColorSpace,_T("ColorSpaceConverter")));

	if (m_out == OUT_IN_AVI){
		// Add AVI MUX
		CComPtr<IBaseFilter> pAviMux;
		JIF(pAviMux.CoCreateInstance(CLSID_AviDest));
		
		JIF(m_pGraphBuilder->AddFilter(pAviMux,_T("AVI Mux")));
		
		// Add File Writer
		CComPtr<IBaseFilter> pFileWriter;
		JIF(pFileWriter.CoCreateInstance(CLSID_FileWriter));
		
		JIF(m_pGraphBuilder->AddFilter(pFileWriter,_T("File Writer")));
		
		// Set File name in File Writer filter
		CComPtr <IFileSinkFilter> pfsink;
		JIF(pFileWriter->QueryInterface(IID_IFileSinkFilter,(void**)&pfsink));
		JIF(pfsink->SetFileName((LPCOLESTR)m_avi.GetBuffer(),NULL));
	}
	//Bind pins
	HRESULT hres;
	CComPtr<IEnumPins> pEnumS;
	JIF(pSource->EnumPins(&pEnumS));
	IPin* pPinS = NULL;
	if(S_OK == pEnumS->Next(1, &pPinS,NULL))
	{
		CComPtr<IEnumPins> pEnumCSC;
		if(S_OK == pColorSpace->EnumPins(&pEnumCSC))
		{
			IPin* pPinCSC[2] = {0};	ULONG fetched;
			hres = pEnumCSC->Next(2, &pPinCSC[0],&fetched);
			if(fetched==2)
			{  hres = m_pGraphBuilder->ConnectDirect(pPinS,pPinCSC[0],NULL);
			   hres = m_pGraphBuilder->Render(pPinCSC[1]);
			   pPinCSC[0]->Release();
			   pPinCSC[1]->Release();
			}
			pPinS->Release();
		}
	}
	JIF(hres);
	
	JIF(m_pGraphBuilder->QueryInterface(IID_IMediaEventEx, (void **)&m_pMediaEvent));
	JIF(m_pMediaEvent->SetNotifyWindow((OAHWND)m_hWnd, WM_GRAPHNOTIFY, 0));
	
	//////////////////////
	//  Run graph
	//////////////////////
	JIF(m_pGraphBuilder->QueryInterface(IID_IMediaControl,(void**)&m_pMediaControl));
	JIF(m_pMediaControl->Run());
	GetDlgItem(IDOK)->SetWindowText(_T("Stop"));
	
	return S_OK;
}

LRESULT CBMP2AVIDlg::onGraphNotify(WPARAM wp, LPARAM)
{
	if (!m_pMediaEvent)
		return 0;
	if (wp){
		ReleaseAll();
		GetDlgItem(IDOK)->SetWindowText(_T("Start"));
		return 0;
	}
	LONG evCode, evParam1, evParam2;
	
	while(SUCCEEDED(m_pMediaEvent->GetEvent(&evCode, &evParam1, &evParam2, 0)))
	{
		BOOL flag = FALSE;
		m_pMediaEvent->FreeEventParams(evCode, evParam1, evParam2);
		switch(evCode) {
		case EC_COMPLETE:
		case EC_USERABORT:
			ReleaseAll();
			flag = TRUE;
			GetDlgItem(IDOK)->SetWindowText(_T("Start"));
			break;
//		default:
		}
		if(flag) break;
		
	}
	return 0;
}


















