// PicturePropPage.cpp: implementation of the CPicturePropPage class.
// Property Page dialog for BMP2Stream filter
// Set Frame rate 
//
// Copyright : Yusupova Julia;  email: julus@yandex.ru
//////////////////////////////////////////////////////////////////////
#include <streams.h>
#include "PictureGuids.h"

#include "PicturePropPage.h"
#include "resource.h"

#include <stdio.h>

CPicturePropPage::CPicturePropPage(LPUNKNOWN pUnk, HRESULT *phr) :
    CBasePropertyPage(NAME("Settings"), pUnk,IDD_PROPERTY_PAGE,IDS_PROPPAGE),
	m_pIPicSettings(NULL),
	m_nFrameRate(20)
{
    ASSERT(phr);
} 

STDMETHODIMP CPicturePropPage::NonDelegatingQueryInterface(REFIID riid, void **ppv)
{
    CheckPointer(ppv,E_POINTER);
	return CBasePropertyPage::NonDelegatingQueryInterface(riid, ppv);
} // NonDelegatingQueryInterface



CUnknown *CPicturePropPage::CreateInstance(LPUNKNOWN lpunk, HRESULT *phr)
{
    CUnknown *punk = new CPicturePropPage(lpunk, phr);
    if (punk == NULL) {
		*phr = E_OUTOFMEMORY;
    }
    return punk;

} // CreateInstance

/////////////
// Called on dialog creation
HRESULT CPicturePropPage::OnActivate()
{

	HWND hwnd = GetDlgItem(m_Dlg, IDC_FRAME_RATE);
	WCHAR str[21]={0};
	wsprintf(str, L"%d", m_nFrameRate);
	SetWindowText(hwnd, str);
	SendMessage(hwnd, EM_SETLIMITTEXT, 6, 0);
		
	return NOERROR;
} 

BOOL CPicturePropPage::OnReceiveMessage(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg)
    {
	case WM_COMMAND:
		{
			switch (LOWORD(wParam))	{
				
			case IDC_FRAME_RATE:
				if(HIWORD(wParam)==EN_CHANGE) {
					m_bDirty = TRUE;
					m_pPageSite->OnStatusChange(PROPPAGESTATUS_DIRTY);
				}
				break;
			};
        }//end case WM_COMMAND
	} 
    return CBasePropertyPage::OnReceiveMessage(hwnd,uMsg,wParam,lParam);
}

// Give us the filter to communicate with
HRESULT CPicturePropPage::OnConnect(IUnknown *pUnknown)
{
    ASSERT(m_pIPicSettings == NULL);

  	HRESULT hr = pUnknown->QueryInterface(IID_IPictureSettings, (void **) &m_pIPicSettings);
    if (FAILED(hr)) 
        return E_NOINTERFACE;
	ASSERT(m_pIPicSettings);
	m_pIPicSettings->get_FrameRate(&m_nFrameRate);
	
	return NOERROR;
} 

// Release the interface
HRESULT CPicturePropPage::OnDisconnect()
{
	if (m_pIPicSettings == NULL) 
        return E_UNEXPECTED;

	m_pIPicSettings->Release();
	m_pIPicSettings = NULL;
	
	return NOERROR;
} 

HRESULT CPicturePropPage::OnApplyChanges()
{
	HWND hwnd = GetDlgItem(m_Dlg, IDC_FRAME_RATE);
	WCHAR str[21]={0};
	GetWindowText(hwnd, str, 20);
	m_nFrameRate = _wtoi(str);
	if (m_nFrameRate==0){
		MessageBox(NULL, L"The frame rate cannot be zero.",L"Error", MB_OK);
		return E_FAIL;
	}
	m_pIPicSettings->put_FrameRate(m_nFrameRate);
	return S_OK;
}
