//------------------------------------------------------------------------------
// File: FPicture.cpp
//
// Desc: DirectShow sample code - implementation of filter behaviors
//       for the BMP2Stream source filter.  
//
// Copyright : Yusupova Julia;  email: julus@yandex.ru
//------------------------------------------------------------------------------
#include <streams.h>
#include <olectl.h>
#include <initguid.h>

#include "PictureGUIDs.h"
#include "fPicture.h"
#include "PicturePropPage.h"

// Setup data

const AMOVIESETUP_MEDIATYPE sudOpPinTypes =
{
    &MEDIATYPE_Video,       // Major type
    &MEDIASUBTYPE_NULL      // Minor type
};

const AMOVIESETUP_PIN sudOpPin =
{
    L"Output",              // Pin string name
    FALSE,                  // Is it rendered
    TRUE,                   // Is it an output
    FALSE,                  // Can we have none
    FALSE,                  // Can we have many
    &CLSID_NULL,            // Connects to filter
    NULL,                   // Connects to pin
    1,                      // Number of types
    &sudOpPinTypes };       // Pin details

const AMOVIESETUP_FILTER sudPictureax =
{
    &CLSID_BMP2Stream,    // Filter CLSID
    L"BMP2Stream",       // String name
    MERIT_DO_NOT_USE,       // Filter merit
    1,                      // Number pins
    &sudOpPin               // Pin details
};


// COM global table of objects in this dll

CFactoryTemplate g_Templates[] = {
	  { L"BMP2Stream"
	  , &CLSID_BMP2Stream
	  , CBMP2Stream::CreateInstance
	  , NULL
	  , &sudPictureax }
  ,
	  { L"BMP2Stream Property Page"
	  , &CLSID_BMP2StreamPropPage
	  , CPicturePropPage::CreateInstance }
};
int g_cTemplates = sizeof(g_Templates) / sizeof(g_Templates[0]);


//---------------------------------------------------------------------

//
// DllRegisterServer
//
// Handles DLL registry
//
STDAPI DllRegisterServer()
{
    return AMovieDllRegisterServer2(TRUE);

} // DllRegisterServer


//
// DllUnregisterServer
//
STDAPI DllUnregisterServer()
{
    return AMovieDllRegisterServer2(FALSE);

} // DllUnregisterServer


//
// DllEntryPoint
//
extern "C" BOOL WINAPI DllEntryPoint(HINSTANCE, ULONG, LPVOID);

BOOL APIENTRY DllMain(HANDLE hModule, 
                      DWORD  dwReason, 
                      LPVOID lpReserved)
{
    return DllEntryPoint((HINSTANCE)(hModule), dwReason, lpReserved);
}

//
// CreateInstance
//
CUnknown * WINAPI CBMP2Stream::CreateInstance(LPUNKNOWN lpunk, HRESULT *phr)
{
    CUnknown *punk = new CBMP2Stream(lpunk, phr);
    if (punk == NULL) {
        *phr = E_OUTOFMEMORY;
    }
    return punk;

} // CreateInstance


//
// Constructor
//
// Initialise a CPictureStream object so that we have a pin.
//
CBMP2Stream::CBMP2Stream(LPUNKNOWN lpunk, HRESULT *phr) :
    CSource(NAME("BMP2Stream"),
            lpunk,
            CLSID_BMP2Stream)
{
	m_pFileName=NULL;
    CAutoLock cAutoLock(&m_cStateLock);

    m_paStreams    = (CSourceStream **) new CPictureStream*[1];
    if (m_paStreams == NULL) {
        *phr = E_OUTOFMEMORY;
	return;
    }

    m_paStreams[0] = new CPictureStream(phr, this, L"BMP2Stream");
    if (m_paStreams[0] == NULL) {
        *phr = E_OUTOFMEMORY;
	return;
    }
	
} // (Constructor)

STDMETHODIMP CBMP2Stream::Load(LPCOLESTR lpwszFileName, const AM_MEDIA_TYPE *pmt)
{
	CheckPointer(lpwszFileName, E_POINTER);

	// lstrlenW is one of the few Unicode functions that works on win95
	int cch = lstrlenW(lpwszFileName) + 1;
	TCHAR *lpszFileName;
	#ifndef UNICODE
	lpszFileName = new char[cch * 2];
	if (!lpszFileName) {
		return E_OUTOFMEMORY;
	}
	WideCharToMultiByte(GetACP(), 0, lpwszFileName, -1,
		lpszFileName, cch, NULL, NULL);
	#else
	lpszFileName = (TCHAR *)lpwszFileName;
	#endif
	CAutoLock cAutoLock(&m_cStateLock);

	//open this file
	HANDLE hFile = CreateFile(lpszFileName,	GENERIC_READ, FILE_SHARE_READ, NULL,
		OPEN_EXISTING, 0, NULL);
    if (hFile == INVALID_HANDLE_VALUE ) {
		MessageBox(NULL,L"Cannot open file", L"Error", MB_OK|MB_ICONSTOP);
#ifndef UNICODE
		delete [] lpszFileName;
#endif
		return E_FAIL;
    }
	if (FALSE==((CPictureStream*)(m_paStreams[0]))->LoadBMP(hFile)) {
		CloseHandle(hFile);
		MessageBox(NULL,L"Cannot open file", L"Error", MB_OK|MB_ICONSTOP);
#ifndef UNICODE
		delete [] lpszFileName;
#endif
		return E_FAIL;
    }
	CloseHandle(hFile);
	
	m_pFileName = new WCHAR[cch];
	if (m_pFileName!=NULL) {
		CopyMemory(m_pFileName, lpwszFileName, cch*sizeof(WCHAR));
	}
	#ifndef UNICODE
		delete [] lpszFileName;
	#endif
	
return S_OK;
}
  
STDMETHODIMP CBMP2Stream::GetCurFile(LPOLESTR * ppszFileName, AM_MEDIA_TYPE *pmt)
{
	CheckPointer(ppszFileName, E_POINTER);
	*ppszFileName = NULL;
	if (m_pFileName!=NULL) {
		DWORD n = sizeof(WCHAR)*(1+lstrlenW(m_pFileName));

		*ppszFileName = (LPOLESTR) CoTaskMemAlloc( n );
		if (*ppszFileName!=NULL) {
			CopyMemory(*ppszFileName, m_pFileName, n);
		}
	}
	pmt = NULL;

return NOERROR;
}


//
// GetPages
//
STDMETHODIMP CBMP2Stream::GetPages(CAUUID * pPages) {
	
    pPages->cElems = 1;
    pPages->pElems = (GUID *) CoTaskMemAlloc(sizeof(GUID));
    if (pPages->pElems == NULL) {
        return E_OUTOFMEMORY;
    }
    *(pPages->pElems) = CLSID_BMP2StreamPropPage;
	
    return NOERROR;
}


//
// get_FrameRate
//
STDMETHODIMP CBMP2Stream::get_FrameRate(int *FrameRate)
{
	((CPictureStream*)(m_paStreams[0]))->GetFrameRate(FrameRate);
	return NOERROR;
}
//
// put_FrameRate
//
STDMETHODIMP CBMP2Stream::put_FrameRate(int FrameRate) 
{
	((CPictureStream*)(m_paStreams[0]))->SetFrameRate(FrameRate);
    return NOERROR;
}


//
// Constructor
//
CPictureStream::CPictureStream(HRESULT *phr,
                         CBMP2Stream *pParent,
                         LPCWSTR pPinName) :
    CSourceStream(NAME("BMP2Stream"),phr, pParent, pPinName),
    m_iRepeatTime(20)
{
    CAutoLock cAutoLock(&m_cSharedState);

    m_pBitmap = NULL;
	m_palette = NULL;
	m_nColor = 0;
	m_iFrameRate = 1000/m_iRepeatTime;
	
	m_rtSampleTime = 0;

} // (Constructor)


//
// Destructor
//
CPictureStream::~CPictureStream()
{
    CAutoLock cAutoLock(&m_cSharedState);
    ClearBitmap();
} // (Destructor)


//
// FillBuffer
//
// Plots a Picture into the supplied video buffer
//
HRESULT CPictureStream::FillBuffer(IMediaSample *pms)
{
    BYTE *pData;
    long lDataLen;

    pms->GetPointer(&pData);
    lDataLen = pms->GetSize();

    // Clear Memory
    ZeroMemory( pData, lDataLen );
    
	CAutoLock cAutoLockShared(&m_cSharedState);

	CopyMemory(pData, m_pBitmap, lDataLen);

    // The current time is the sample's start
    CRefTime rtStart = m_rtSampleTime;

    // Increment to find the finish time
    m_rtSampleTime += (LONG)m_iRepeatTime;

    pms->SetTime((REFERENCE_TIME *) &rtStart,(REFERENCE_TIME *) &m_rtSampleTime);

    //pms->SetSyncPoint(TRUE);
    return S_OK;

} // FillBuffer


//
// GetMediaType
//
HRESULT CPictureStream::GetMediaType(CMediaType *pmt)
{
	CAutoLock cAutoLock(m_pFilter->pStateLock());
    
    VIDEOINFO *pvi = (VIDEOINFO *) pmt->AllocFormatBuffer(sizeof(VIDEOINFO));
    if (NULL == pvi) {
		return(E_OUTOFMEMORY);
    }
    ZeroMemory(pvi, sizeof(VIDEOINFOHEADER));
	
    pvi->bmiHeader = m_bmpHeader;
	
	if (m_palette){
		for (int i = 0; i < m_nColor; i++) {
			pvi->bmiColors[i].rgbRed = m_palette[i].rgbRed;
			pvi->bmiColors[i].rgbBlue = m_palette[i].rgbBlue;
			pvi->bmiColors[i].rgbGreen = m_palette[i].rgbGreen;
			pvi->bmiColors[i].rgbReserved = 0;
		}
	}
    
    SetRectEmpty(&(pvi->rcSource));	// we want the whole image area rendered.
    SetRectEmpty(&(pvi->rcTarget));	// no particular destination rectangle

    pmt->SetType(&MEDIATYPE_Video);
    pmt->SetFormatType(&FORMAT_VideoInfo);
    pmt->SetTemporalCompression(FALSE);

    // Work out the GUID for the subtype from the header info.
    const GUID SubTypeGUID = GetBitmapSubtype(&pvi->bmiHeader);
	
	//pmt->SetSubtype(&MEDIASUBTYPE_RGB24);
    pmt->SetSubtype(&SubTypeGUID);
    pmt->SetSampleSize(pvi->bmiHeader.biSizeImage);

    return NOERROR;
}


//
// CheckMediaType
//
// We will accept video formats, in appropriate image size and picture format
// Returns E_INVALIDARG if the mediatype is not acceptable
//
HRESULT CPictureStream::CheckMediaType(const CMediaType *pMediaType)
{
    CAutoLock cAutoLock(m_pFilter->pStateLock());

    if ((*(pMediaType->Type()) != MEDIATYPE_Video)	// we only output video!
	    || !(pMediaType->IsFixedSize()) ) {		// ...in fixed size samples
                return E_INVALIDARG;
    }

    // Check for the subtypes we support
    const GUID *SubType = pMediaType->Subtype();
    if (*SubType != GetBitmapSubtype(&m_bmpHeader) )
	{
                return E_INVALIDARG;
    }

    // Get the format area of the media type
    VIDEOINFO *pvi = (VIDEOINFO *) pMediaType->Format();

    if (pvi == NULL)
	return E_INVALIDARG;

    // Check the image size. As my default ball is 10 pixels big
    // look for at least a 20x20 image. This is an arbitary size constraint,
    // but it avoids balls that are bigger than the picture...

    if ((pvi->bmiHeader.biWidth < m_bmpHeader.biWidth ) || 
		(pvi->bmiHeader.biHeight < m_bmpHeader.biHeight) ) {
	return E_INVALIDARG;
    }

    
    return S_OK;  // This format is acceptable.

} // CheckMediaType


HRESULT CPictureStream::SetFrameRate(INT nFrameRate)
{
	CAutoLock cAutoLockShared(&m_cSharedState);
	m_iFrameRate = nFrameRate;
	m_iRepeatTime = 1000/nFrameRate;

	return S_OK;		
}


//
// DecideBufferSize
//
// This will always be called after the format has been sucessfully
// negotiated. So we have a look at m_mt to see what size image we agreed.
// Then we can ask for buffers of the correct size to contain them.
//
HRESULT CPictureStream::DecideBufferSize(IMemAllocator *pAlloc,ALLOCATOR_PROPERTIES *pProperties)
{
    CAutoLock cAutoLock(m_pFilter->pStateLock());
    ASSERT(pAlloc);
    ASSERT(pProperties);
    HRESULT hr = NOERROR;

    VIDEOINFO *pvi = (VIDEOINFO *) m_mt.Format();
    pProperties->cBuffers = 1;
    pProperties->cbBuffer = pvi->bmiHeader.biSizeImage;

    ASSERT(pProperties->cbBuffer);

    // Ask the allocator to reserve us some sample memory, NOTE the function
    // can succeed (that is return NOERROR) but still not have allocated the
    // memory that we requested, so we must check we got whatever we wanted

    ALLOCATOR_PROPERTIES Actual;
    hr = pAlloc->SetProperties(pProperties,&Actual);
    if (FAILED(hr)) {
        return hr;
    }

    // Is this allocator unsuitable

    if (Actual.cbBuffer < pProperties->cbBuffer) {
        return E_FAIL;
    }

    // Make sure that we have only 1 buffer (we erase the Picture in the
    // old buffer to save having to zero a 200k+ buffer every time
    // we draw a frame)

    ASSERT( Actual.cBuffers == 1 );
    return NOERROR;

} // DecideBufferSize



//
// OnThreadCreate
// As we go active reset the stream time to zero
HRESULT CPictureStream::OnThreadCreate()
{
    CAutoLock cAutoLockShared(&m_cSharedState);
    m_rtSampleTime = 0;

    return NOERROR;
} // OnThreadCreate


//////////////////////////////////////////////////////////////////////////
#define SAFE_RETURN_FALSE {delete [] pbMem; return FALSE;}

BOOL CPictureStream::LoadBMP(HANDLE hFile)
{
	CAutoLock cAutoLockShared(&m_cSharedState);
	
	BITMAPINFOHEADER bmpHeader = {0};
	
	// File header contain 14 bytes and Information header is 50 bytes
	DWORD dwBytes = 54;
	PBYTE pbMem = new BYTE[dwBytes];
	PBYTE pData = pbMem;
	if (!pbMem) return FALSE;
	DWORD dwBytesRead;
	//BMP_content cont={0};
	if (!ReadFile(hFile, (LPVOID)pbMem, dwBytes, &dwBytesRead, NULL) || dwBytesRead != dwBytes) {
        SAFE_RETURN_FALSE;
    }
	//File header (14 bytes)
	// 1 - BM signature, "BM" = 0x4d42 (2 bytes)
	//if (strncmp((PCHAR)pData, "BM", 2)) SAFE_RETURN_FALSE;
	if (*((PWORD)pData) != 0x4d42) SAFE_RETURN_FALSE;
	pData += 2;
	// 2 - File size (4 bytes)
	pData += 4;
	// 3, 4 - Reserved (4 bytes)
	pData += 4;
	//5 - Location of bitmap data (4 bytes)
	DWORD location_bitmap_data = *((PDWORD)pData);
	pData += 4;
	
	// Information header (50 bytes)
	// 1 - size of information header (4 bytes) = 40
	if (40 != *((PDWORD)pData)) SAFE_RETURN_FALSE;
	pData += 4;
	bmpHeader.biSize = 40;
	// 2 - Image width (4 bytes)
	bmpHeader.biWidth = *((PLONG)pData);
	pData += 4;
	if (bmpHeader.biWidth<=0) SAFE_RETURN_FALSE;
	// 3 - Image height (4 bytes)
	bmpHeader.biHeight = *((PLONG)pData);
	pData += 4;
	if (bmpHeader.biHeight<=0) SAFE_RETURN_FALSE;
	// 4 - Number of color planes (2 bytes) = 1
	if (1 != *((PWORD)pData)) SAFE_RETURN_FALSE;
	pData += 2;
	bmpHeader.biPlanes = 1;
	// 5 - Number of bits per pixel (2 bytes)
	bmpHeader.biBitCount = *((PWORD)pData);
	pData += 2;
	if ((!(bmpHeader.biBitCount==1 ||
		bmpHeader.biBitCount==4 ||
		bmpHeader.biBitCount==8)) &&
		bmpHeader.biBitCount<=8) 
		SAFE_RETURN_FALSE;
	// 6 - Compression method used (4 bytes)
	bmpHeader.biCompression = *((PDWORD)pData);
	pData += 4;
	// this program don`t work with RLE compression files
	if (bmpHeader.biCompression != BI_RGB) SAFE_RETURN_FALSE; 
	// 7 - Number of bytes in bitmap data (4 bytes)
	bmpHeader.biSizeImage = *((PDWORD)pData);
	pData += 4;
	// 8 - Horisontal screen resolution (4 bytes)
	bmpHeader.biXPelsPerMeter = *((PLONG)pData);
	pData += 4;
	// 9 - Vertical screen resolution (4 bytes)
	bmpHeader.biYPelsPerMeter = *((PLONG)pData);
	pData += 4;
	// 10 - Number of colors used in the pictures (4 bytes)
	bmpHeader.biClrUsed = *((PDWORD)pData);
	pData += 4;
	// 11 - Number of important colors (4 bytes)
	bmpHeader.biClrImportant = *((PDWORD)pData);
	pData += 4;

	delete [] pbMem;

	if (!bmpHeader.biSizeImage)
		bmpHeader.biSizeImage = GetBitmapSize(&bmpHeader);

	ClearBitmap();

	// Read palette
	if (bmpHeader.biBitCount <= 8){
		if (bmpHeader.biClrUsed)
			m_nColor = bmpHeader.biClrUsed;
		else{
			switch(bmpHeader.biBitCount) {
			case 1: m_nColor = 2; // (2^1)
				break;
			case 4: m_nColor = 16; // (2^4)
				break;
			case 8: m_nColor = 256; // (2^8)
				break;
			}
		}
		dwBytes = m_nColor * 4;
		pbMem = new BYTE[dwBytes];
		pData = pbMem;
		// Read palette from file
		if (!ReadFile(hFile, (LPVOID)pbMem, dwBytes, &dwBytesRead, NULL) || dwBytesRead != dwBytes)
		    SAFE_RETURN_FALSE;
		m_palette = new RGBQUAD[m_nColor];
		for (int i=0; i<m_nColor; i++){
			m_palette[i].rgbBlue = *(pData++);
			m_palette[i].rgbGreen = *(pData++);
			m_palette[i].rgbRed = *(pData++);
			m_palette[i].rgbReserved = *(pData++);
		}
		delete [] pbMem;
    }
	
	// Read bitmap
	dwBytes = bmpHeader.biSizeImage;
	pbMem = new BYTE[dwBytes];
	
	if (!ReadFile(hFile, (LPVOID)pbMem, dwBytes, &dwBytesRead, NULL) || dwBytesRead != dwBytes)
		SAFE_RETURN_FALSE;
	m_pBitmap = pbMem;
	m_bmpHeader = bmpHeader;
	return TRUE;
}
//////////////////////////////////////////////////////////////////////////


void CPictureStream::ClearBitmap()
{
	if (m_pBitmap){
		delete [] m_pBitmap;
		m_pBitmap = NULL;
	}
	if (m_palette){
		delete [] m_palette;
		m_palette = NULL;
	}
	m_nColor = 0;
}
