//------------------------------------------------------------------------------
// File: FPicture.h
//
// Desc: DirectShow sample code - main header file for the BMP2Stream filter. 
// For more information refer to FPicture.cpp
//
// Copyright : Yusupova Julia;  email: julus@yandex.ru
//------------------------------------------------------------------------------


#include <stdio.h>

//------------------------------------------------------------------------------
// Class CBMP2Stream
//
// This is the main class for the BMP2Stream filter. It inherits from
// CSource, the DirectShow base class for source filters.
//------------------------------------------------------------------------------
class CBMP2Stream: public IFileSourceFilter, 
						public CSource,
						public ISpecifyPropertyPages,
						public IPictureSettings
{
public:

	static CUnknown * WINAPI CreateInstance(LPUNKNOWN lpunk, HRESULT *phr);

	DECLARE_IUNKNOWN
		
	STDMETHODIMP NonDelegatingQueryInterface(REFIID riid, void **ppv)
    {
        if (riid == IID_IFileSourceFilter) {
            return GetInterface((IFileSourceFilter *)this, ppv);
        } 
		else if (riid == IID_ISpecifyPropertyPages) {
			return GetInterface((ISpecifyPropertyPages *) this, ppv);
		}
		else if (riid == IID_IPictureSettings) {
			return GetInterface((IPictureSettings *) this, ppv);
		}
		else {
            return CSource::NonDelegatingQueryInterface(riid, ppv);
        }
    }
	
    //  IFileSourceFilter methods 
    //  Load a (new) file
	// Modelled on IPersistFile::Load
    // Caller needs to CoTaskMemFree or equivalent.
	STDMETHODIMP Load(LPCOLESTR lpwszFileName, const AM_MEDIA_TYPE *pmt);
    STDMETHODIMP GetCurFile(LPOLESTR * ppszFileName, AM_MEDIA_TYPE *pmt);
    
	~CBMP2Stream(){
		if (m_pFileName)
			delete [] m_pFileName;
	}

	// return our property pages
    STDMETHODIMP GetPages(CAUUID * pPages);

	// Implement IPictureSettings methods
	STDMETHODIMP get_FrameRate(int *FrameRate);
    STDMETHODIMP put_FrameRate(int FrameRate);
	
private:
	
	LPWSTR     m_pFileName;
    // It is only allowed to to create these objects with CreateInstance
    CBMP2Stream(LPUNKNOWN lpunk, HRESULT *phr);

}; // CBMP2Stream


//------------------------------------------------------------------------------
// Class CPictureStream
//
// This class implements the stream which is used to output the BMP
// data from the source filter. It inherits from DirectShows's base
// CSourceStream class.
//------------------------------------------------------------------------------
class CPictureStream : public CSourceStream
{

public:

    CPictureStream(HRESULT *phr, CBMP2Stream *pParent, LPCWSTR pPinName);
    ~CPictureStream();

    // plots a Picture into the supplied video frame
    HRESULT FillBuffer(IMediaSample *pms);
	
	// draw bmp picture in pFrame
	//BOOL PlotPicture(BYTE pFrame[], LONG nFrame);

    // Ask for buffers of the size appropriate to the agreed media type
    HRESULT DecideBufferSize(IMemAllocator *pIMemAlloc,
                             ALLOCATOR_PROPERTIES *pProperties);

    HRESULT GetMediaType(CMediaType *pmt);

	HRESULT CheckMediaType(const CMediaType *pMediaType);


    // Resets the stream time to zero
    HRESULT OnThreadCreate(void);

    // Quality control notifications sent to us
    STDMETHODIMP Notify(IBaseFilter * pSender, Quality q) { return S_OK;}

	BOOL LoadBMP(HANDLE hFile);
	void ClearBitmap();

	HRESULT SetFrameRate(INT nFrameRate);
	void GetFrameRate(INT *nFrameRate) {
		*nFrameRate = m_iFrameRate;
	}
	
private:

	PBYTE m_pBitmap;
	LONG m_offset;						// Offset in bitmap Data
	RGBQUAD * m_palette;				// Palette for 1,4 and 8 bits bitmap
	INT m_nColor;						// Number of colours used in m_palette
	BITMAPINFOHEADER m_bmpHeader;
	int m_iFrameRate;					// Number of rendered frames for seconds
	int m_iRepeatTime;                  // Time in msec between frames
	CCritSec m_cSharedState;	        // Lock on m_rtSampleTime and m_Picture
	CRefTime m_rtSampleTime;	        // The time stamp for each sample

}; // CPictureStream
	
